import { resolve, relative } from "path";
import module from "module";
const { createRequire } = module;
const require = createRequire(import.meta.url);
import { fileExists } from "./fs.js";
import { createConfiguration } from "snowpack";
import cc from "cosmiconfig";
const { cosmiconfig } = cc;
const _config = require("microsite/assets/snowpack.config.cjs");
const pkg = require(resolve(process.cwd(), "package.json"));
// const deps = Object.keys(
//   pkg.dependencies || {}
// );
const DEFAULT_BASE_PATH = pkg.homepage || '/';
export function resolveNormalizedBasePath(args) {
    var _a;
    let basePath = (_a = args['--base-path']) !== null && _a !== void 0 ? _a : DEFAULT_BASE_PATH;
    return basePath === '/' ? basePath : `/${basePath.replace(/^\//, '').replace(/\/$/, '')}/`;
}
async function hasPostCSSConfig() {
    try {
        const explorer = cosmiconfig("postcss");
        const result = await explorer.search();
        if (result.filepath)
            return true;
    }
    catch (e) { }
    return false;
}
export async function loadConfiguration(mode) {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s;
    const [snowpackconfigPath, tsconfigPath, usesPostCSS] = await Promise.all([
        findSnowpackConfig(),
        findTsOrJsConfig(),
        hasPostCSSConfig(),
    ]);
    const aliases = tsconfigPath
        ? resolveTsconfigPathsToAlias({ tsconfigPath })
        : {};
    const userConfig = snowpackconfigPath ? require(snowpackconfigPath) : {};
    const additionalPlugins = usesPostCSS ? ["@snowpack/plugin-postcss"] : [];
    switch (mode) {
        case "dev":
            return createConfiguration(Object.assign(Object.assign(Object.assign({}, userConfig), _config), { plugins: [...additionalPlugins, ..._config.plugins, ...((_a = userConfig.plugins) !== null && _a !== void 0 ? _a : [])], alias: Object.assign(Object.assign(Object.assign(Object.assign({}, ((_b = userConfig.aliases) !== null && _b !== void 0 ? _b : {})), aliases), ((_c = _config.alias) !== null && _c !== void 0 ? _c : {})), { "microsite/hydrate": "microsite/client/hydrate" }), installOptions: Object.assign(Object.assign(Object.assign({}, ((_d = userConfig.installOptions) !== null && _d !== void 0 ? _d : {})), _config.installOptions), { externalPackage: ["/web_modules/microsite/_error.js"] }) }));
        case "build":
            return createConfiguration(Object.assign(Object.assign(Object.assign({}, userConfig), _config), { plugins: [...additionalPlugins, ..._config.plugins, ...((_e = userConfig.plugins) !== null && _e !== void 0 ? _e : [])], alias: Object.assign(Object.assign(Object.assign({}, ((_f = userConfig.aliases) !== null && _f !== void 0 ? _f : {})), aliases), ((_g = _config.alias) !== null && _g !== void 0 ? _g : {})), installOptions: Object.assign(Object.assign(Object.assign({}, ((_h = userConfig.installOptions) !== null && _h !== void 0 ? _h : {})), _config.installOptions), { rollup: Object.assign(Object.assign(Object.assign({}, ((_k = (_j = userConfig.installOptions) === null || _j === void 0 ? void 0 : _j.rollup) !== null && _k !== void 0 ? _k : {})), ((_m = (_l = _config.installOptions) === null || _l === void 0 ? void 0 : _l.rollup) !== null && _m !== void 0 ? _m : {})), { plugins: [
                            ...((_q = (_p = (_o = userConfig.installOptions) === null || _o === void 0 ? void 0 : _o.rollup) === null || _p === void 0 ? void 0 : _p.plugins) !== null && _q !== void 0 ? _q : []),
                            {
                                name: '@microsite/auto-external',
                                options(opts) {
                                    return Object.assign({}, opts, { external: (source) => source.startsWith('preact') });
                                }
                            }
                        ] }), externalPackage: [
                        ...((_s = (_r = userConfig.installOptions) === null || _r === void 0 ? void 0 : _r.externalPackage) !== null && _s !== void 0 ? _s : []),
                        ..._config.installOptions.externalPackage,
                    ].filter((v) => v !== "preact") }) }));
    }
}
const findSnowpackConfig = async () => {
    const cwd = process.cwd();
    const snowpack = resolve(cwd, './snowpack.config.cjs');
    if (await fileExists(snowpack))
        return snowpack;
    return null;
};
const findTsOrJsConfig = async () => {
    const cwd = process.cwd();
    const tsconfig = resolve(cwd, "./tsconfig.json");
    if (await fileExists(tsconfig))
        return tsconfig;
    const jsconfig = resolve(cwd, "./jsconfig.json");
    if (await fileExists(jsconfig))
        return jsconfig;
    return null;
};
function resolveTsconfigPathsToAlias({ tsconfigPath = "./tsconfig.json", } = {}) {
    var _a, _b;
    let { baseUrl, paths } = (_b = (_a = require(tsconfigPath)) === null || _a === void 0 ? void 0 : _a.compilerOptions) !== null && _b !== void 0 ? _b : {};
    if (!(baseUrl && paths))
        return {};
    baseUrl = resolve(process.cwd(), baseUrl);
    const aliases = {};
    Object.keys(paths).forEach((item) => {
        const key = item.replace("/*", "");
        const value = "./" +
            relative(process.cwd(), resolve(baseUrl, paths[item][0].replace("/*", "").replace("*", "")));
        aliases[key] = value;
    });
    return aliases;
}
